package mdut

import (
	"fmt"
	"io"
	"net/http"

	"golang.org/x/net/html"
)

/* https://siongui.github.io/2016/05/10/go-get-html-title-via-net-html/ */
func isTitleElement(n *html.Node) bool {
	return n.Type == html.ElementNode && n.Data == "title"
}

func traverse(n *html.Node) (string, bool) {
	if isTitleElement(n) {
		return n.FirstChild.Data, true
	}

	for c := n.FirstChild; c != nil; c = c.NextSibling {
		result, ok := traverse(c)
		if ok {
			return result, ok
		}
	}

	return "", false
}

func getHtmlTitle(r io.Reader) string {
	doc, err := html.Parse(r)
	if err != nil {
		panic("Fail to parse html")
	}

	title, _ := traverse(doc)
	return title
}

/* Done From https://siongui.github.io/2016/05/10/go-get-html-title-via-net-html/ */
func GetTitle(url string) (string, error) {
	res, err := http.Get(url)
	if err != nil {
		return "", err
	}

	defer res.Body.Close()
	if res.StatusCode != 200 {
		return "", fmt.Errorf("status code error: %d %s", res.StatusCode, res.Status)
	}
	title := getHtmlTitle(res.Body)
	return title, nil

}
